@@ -103,12 +103,8 @@ class Max(_Aggregation):
         self.source = source
 
     def oninsert(self, aggregator, instance):
-        if aggregator.mapper.local_table.metadata.bind.url.drivername == 'mysql':
-            return {self.target.name: if_func((func.ifnull(self.target,0) < getattr(instance, self.source.name)),
-                getattr(instance, self.source.name), self.target)}
-        else:
-            return {self.target.name: func.max(func.ifnull(self.target,0), getattr(instance, self.source.name))}
-
+        return { self.target.name: aggregator.max(self.target, getattr(instance, self.source.name)) }
+        
     def onrecalc(self, aggregator, instance, old=False):
         return {self.target.name: select([func.max(self.source)],
             self.key.parent == self._instanceattr(instance, self.attribute, old))}
@@ -117,7 +113,7 @@ class Max(_Aggregation):
         if getattr(instance, self.source.name) >= self._orig(instance, self.source.name):
             return self.oninsert(aggregator, instance)
         else:
-            return self.onupdate(aggregator, instance)
+            return self.onrecalc(aggregator, instance, False)
 
     def ondelete(self, aggregator, instance):
         return self.onrecalc(aggregator, instance, True)
@@ -138,11 +134,7 @@ class Min(_Aggregation):
         self.source = source
 
     def oninsert(self, aggregator, instance):
-        if aggregator.mapper.local_table.metadata.bind.url.drivername == 'mysql':
-            return {self.target.name: if_func(func.ifnull(self.target,sys.maxint) > getattr(instance, self.source.name),
-                getattr(instance, self.source.name), self.target)}
-        else:
-            return {self.target.name: func.min(func.ifnull(self.target,sys.maxint), getattr(instance, self.source.name))}
+        return { self.target.name: aggregator.min(self.target, getattr(instance, self.source.name)) }
 
     def onrecalc(self, aggregator, instance, old=False):
         return {self.target.name: select([func.min(self.source)],
@@ -152,7 +144,7 @@ class Min(_Aggregation):
         if getattr(instance, self.source.name) <= self._orig(instance, self.source.name):
             return self.oninsert(aggregator, instance)
         else:
-            return self.onupdate(aggregator, instance)
+            return self.onrecalc(aggregator, instance, False)
 
     def ondelete(self, aggregator, instance):
         return self.onrecalc(aggregator, instance, True)
@@ -251,6 +243,18 @@ class Quick(MapperExtension):
                     updates_insert.update(getattr(f, self._insert_method)(self, instance))
                 table.update(condition_delete, values=updates_delete).execute()
                 table.update(condition_insert, values=updates_insert).execute()
+    
+    def max(self, a, b):
+        if self.mapper.local_table.metadata.bind.url.drivername == 'mysql':
+            return if_func((a == None)|(a < b), b, a)
+        else:
+            return func.max(func.ifnull(a,b-1), b)
+
+    def min(self, a, b):
+        if self.mapper.local_table.metadata.bind.url.drivername == 'mysql':
+            return if_func((a == None)|(a > b), b, a)
+        else:
+            return func.min(func.ifnull(a,b+1), b)
 
 class Accurate(Quick):
     """Mapper extension which maintains aggregations
