@@ -10,7 +10,7 @@ import unittest
 import aggregator as a
 from sqlalchemy import (
     Column, ForeignKey,
-    Integer, String,
+    Integer, String, Numeric,
     Table, MetaData,
     )
 from sqlalchemy.orm import (
@@ -69,9 +69,6 @@ class SimpleTest(unittest.TestCase):
         l = self.Line()
         l.block = b.id
         self.save(l)
-    
-    def testUpdate(self):
-        self.blocks.update(values={'lines': self.blocks.c.lines + 1}).execute()
 
     def testAddLines(self):
         b = self.Block()
@@ -120,7 +117,6 @@ class SimpleTest(unittest.TestCase):
         self.session.refresh(b)
         self.assertEquals(b.lines, 11)
         self.assertEquals(b.length, 70)
-        
 
     def testNULL(self):
         b = self.Block()
@@ -595,6 +591,144 @@ class RelationsTest(unittest.TestCase):
         self.assertEquals(m.blocks, 2)
         self.assertEquals(j.lines, 13)
         self.assertEquals(m.lines, 7)
+        
+class TestBigValue(unittest.TestCase):
+    def __init__(self, arg):
+        self.aggregator_class = a.Quick
+        return super(TestBigValue, self).__init__(arg)
+
+    def setUp(self):
+        meta = self.meta = MetaData(bind=dburl)
+        self.session = create_session()
+        blocks = Table('blocks', meta,
+            Column('id', Integer, primary_key=True, autoincrement=True),
+            Column('firstline', Numeric(len(str(sys.maxint))+1, 0)),
+            )
+        lines = Table('lines', meta,
+            Column('id', Numeric(len(str(sys.maxint))+1, 0), primary_key=True, autoincrement=True),
+            Column('block', Integer, ForeignKey(blocks.c.id)),
+            )
+        class Block(object):
+            pass
+        class Line(object):
+            pass
+        self.Block = Block
+        self.Line = Line
+        self.blocks = blocks
+        self.lines = lines
+        blocks.create()
+        lines.create()
+        mapper(Block, blocks)
+        mapper(Line, lines,
+            extension=self.aggregator_class(
+                a.Min(blocks.c.firstline, lines.c.id),
+            ))
+
+    def tearDown(self):
+        self.lines.drop()
+        self.blocks.drop()
+        
+    def save(self, ob):
+        self.session.save(ob)
+        self.session.flush()
+
+    def testBigValue(self):
+        b = self.Block()
+        b.firstline = None
+        self.save(b)
+        l = self.Line()
+        l.id = sys.maxint + 2
+        l.block = b.id
+        self.save(l)
+        self.session.refresh(b)
+        self.assertEquals(b.firstline, sys.maxint+2)
+
+    def testSmallValue(self):
+        b = self.Block()
+        b.firstline = None
+        self.save(b)
+        l = self.Line()
+        l.id = -1
+        l.block = b.id
+        self.save(l)
+        self.session.refresh(b)
+        self.assertEquals(b.firstline, -1)
+
+class TestUpdates(unittest.TestCase):
+    def __init__(self, arg):
+        self.aggregator_class = a.Quick
+        return super(TestUpdates, self).__init__(arg)
+
+    def setUp(self):
+        meta = self.meta = MetaData(bind=dburl)
+        self.session = create_session()
+        blocks = Table('blocks', meta,
+            Column('id', Integer, primary_key=True, autoincrement=True),
+            Column('minlength', Integer),
+            Column('maxlength', Integer),
+            )
+        lines = Table('lines', meta,
+            Column('id', Integer, primary_key=True, autoincrement=True),
+            Column('block', Integer, ForeignKey(blocks.c.id)),
+            Column('length', Integer),
+            )
+        class Block(object):
+            pass
+        class Line(object):
+            pass
+        self.Block = Block
+        self.Line = Line
+        self.blocks = blocks
+        self.lines = lines
+        blocks.create()
+        lines.create()
+        mapper(Block, blocks)
+        mapper(Line, lines,
+            extension=self.aggregator_class(
+                a.Min(blocks.c.minlength, lines.c.length),
+                a.Max(blocks.c.maxlength, lines.c.length),
+            ))
+
+    def tearDown(self):
+        self.lines.drop()
+        self.blocks.drop()
+        
+    def save(self, ob):
+        self.session.save(ob)
+        self.session.flush()
+    
+    def testAddValue(self):
+        b = self.Block()
+        b.lines = 0
+        self.save(b)
+        for i in range(10):
+            l = self.Line()
+            l.block = b.id
+            l.length = i
+            self.session.save(l)
+        l = self.Line()
+        l.block = b.id
+        l.length = 15
+        self.save(l)
+        self.session.refresh(b)
+        self.assertEquals(b.minlength, 0)
+        self.assertEquals(b.maxlength, 15)
+        l.length = 25
+        self.session.flush()
+        self.session.refresh(b)
+        self.assertEquals(b.minlength, 0)
+        self.assertEquals(b.maxlength, 25)
+        l.length = -10
+        self.session.flush()
+        self.session.refresh(b)
+        self.assertEquals(b.minlength, -10)
+        self.assertEquals(b.maxlength, 9)
+        l.length = 5
+        self.session.flush()
+        self.session.refresh(b)
+        self.assertEquals(b.minlength, 0)
+        self.assertEquals(b.maxlength, 9)
+        
 
 class SimpleTest2(SimpleTest):
     def __init__(self, arg):
@@ -611,5 +745,15 @@ class RelationsTest2(ComplexTest):
         self.aggregator_class = a.Accurate
         return unittest.TestCase.__init__(self, arg)
 
+class TestBigValue2(TestBigValue):
+    def __init__(self, arg):
+        self.aggregator_class = a.Accurate
+        return unittest.TestCase.__init__(self, arg)
+
+class TestUpdates2(TestUpdates):
+    def __init__(self, arg):
+        self.aggregator_class = a.Accurate
+        return unittest.TestCase.__init__(self, arg)
+
 if __name__ == '__main__':
     unittest.main()
